package packages.ui;

import greenfoot.*;
import packages.event.Action;
public class Switch extends packages.ui.Button{
    private GreenfootImage on, off, clickedOn, clickedOff, hoveredOn, hoveredOff;

    private boolean state = false;
    
    public Switch(){
        this(20);
    }

    public Switch(int size){
        super(generateImageOff(size), false);
        addClickAction(info -> switchState());

        on = generateImageOn(size);
        off = generateImageOff(size);
        clickedOn = new GreenfootImage(on);
        clickedOn.setColor(CLICK_COLOR);
        clickedOn.fill();
        clickedOff = new GreenfootImage(off);
        clickedOff.setColor(CLICK_COLOR);
        clickedOff.fill();
        hoveredOn = new GreenfootImage(on);
        hoveredOn.scale((int)(on.getWidth() * 1.08), (int)(on.getHeight() * 1.08));
        hoveredOff = new GreenfootImage(off);
        hoveredOff.scale((int)(off.getWidth() * 1.08), (int)(off.getHeight() * 1.08));

        createAndSetImages();
        updateAnimations();
    }
    
    private static GreenfootImage generateImageOff(int size){
        if(size < 10) size = 10;
        GreenfootImage image = generateBackgroundImage(size);
        image.drawImage(generateHandleImage(Color.RED.darker(), size), 1, 1);
        return image;
    }
    private static GreenfootImage generateImageOn(int size){
        if(size < 10) size = 10;
        GreenfootImage image = generateBackgroundImage(size);
        image.drawImage(generateHandleImage(Color.GREEN, size), size + 1, 1);
        return image;
    }
    private static GreenfootImage generateBackgroundImage(int size){
        GreenfootImage image = new GreenfootImage(2 * size, size);
        image.setColor(Color.LIGHT_GRAY);
        image.fill();
        image.setColor(Color.DARK_GRAY);
        image.drawRect(0, 0, 2 * size - 1, size - 1);
        return image;
    }
    private static GreenfootImage generateHandleImage(Color c, int size){
        GreenfootImage image = new GreenfootImage(size - 2, size - 2);
        image.setColor(c);
        image.fill();
        image.setColor(c.darker());
        image.drawRect(0, 0, size - 3, size - 3);
        return image;
    }
    
    public void setState(boolean state){
        this.state = state;
        createAndSetImages();
        updateAnimations();
    }
    public void switchState(){
        setState(!getState());
        onSwitch(getState());
    }


    @Override
    protected void createAndSetImages() {
        if(state) {
            image = on;
            hoveredImage = hoveredOn;
            clickedImage = clickedOn;
        }
        else {
            image = off;
            hoveredImage = hoveredOff;
            clickedImage = clickedOff;
        }
    }

    @Override
    protected void onClick() {
        for (Action action : clickActions) {
            action.run(state);
        }
    }

    public void onSwitch(boolean state){}
    
    public boolean getState(){
        return state;
    }


    /**
     * Adds a click action. The action contains a boolean with the current state.
     * 
     * @param action The action to add
     */
    @Override
    public void addClickAction(Action action) {
        super.addClickAction(action);
    }
}